#!/usr/bin/env python
# coding: utf-8
from __future__ import division, print_function, unicode_literals
import re
import time
import codecs
import argparse
import sys
from builtins import str
import CommonMark


class colors:
    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'


def trace_calls(frame, event, arg):
    co = frame.f_code
    func_name = co.co_name
    if func_name == "write":
        return
    line_no = frame.f_lineno
    filename = co.co_filename
    if event == "call" and not re.match("__", func_name) and \
       re.search("CommonMark.py", filename) \
       and func_name != "dumpAST":
        print("-> " + frame.f_back.f_code.co_name +
              " at " + str(frame.f_back.f_lineno) +
              " called " + func_name + " at " + str(line_no) +
              " in " + filename)
        return trace_calls

parser = argparse.ArgumentParser(
    description="script to run the CommonMark specification tests " +
    "against the CommonMark.py parser")
parser.add_argument(
    '-t',
    help="Single test to run or comma separated list " +
    "of tests (-t 10 or -t 10,11,12,13)")
parser.add_argument(
    '-p',
    action="store_true",
    help="Print passed test information")
parser.add_argument(
    '-f',
    action="store_true",
    help="Print failed tests (during -np...)")
parser.add_argument(
    '-i',
    action="store_true",
    help="Interactive Markdown input mode")
parser.add_argument(
    '-d',
    action="store_true",
    help="Debug, trace calls")
parser.add_argument(
    '-np',
    action="store_true",
    help="Only print section header, tick, or cross")
parser.add_argument(
    '-s',
    action="store_true",
    help="Print percent of tests passed by category")
args = parser.parse_args()

if args.d:
    sys.settrace(trace_calls)

renderer = CommonMark.HtmlRenderer()
parser = CommonMark.Parser()

f = codecs.open("spec.txt", encoding="utf-8")
datalist = []
for line in f:
    datalist.append(line)
data = "".join(datalist)
passed = 0
failed = 0
catStats = {}
examples = []
example_number = 0
current_section = ""
tabChar = '\u2192'
spaceChar = '\u2423'
nbspChar = '\u00A0'


def showSpaces(t):
        t = re.sub("\\t", tabChar, t)
        t = re.sub(" ", spaceChar, t)
        t = re.sub(nbspChar, spaceChar, t)
        return t

t = re.sub("\r\n", "\n", data)

tests = re.sub("^<!-- END TESTS -->(.|[\n])*", '', t, flags=re.M)
testMatch = re.findall(
    re.compile(
        "^\.\n([\s\S]*?)^\.\n([\s\S]*?)^\.$|^#{1,6} *(.*)$",
        re.M),
    tests)

for match in testMatch:
    if not match[2] == "":
        current_section = match[2]
    else:
        example_number += 1
        examples.append({
            'markdown': match[0],
            'html': match[1],
            'section': current_section,
            'number': example_number})

current_section = ""

startTime = time.clock()

if args.i:
    print(
        colors.OKGREEN +
        "(To end input of Markdown block enter 'end' on " +
        "it's own line, to quit enter 'quit')" +
        colors.ENDC)
    while True:
        s = ""
        while True:
            if sys.version_info >= (3, 0):
                inp = input(colors.OKBLUE + 'Markdown: ' + colors.ENDC)
            else:
                inp = raw_input(colors.OKBLUE + 'Markdown: ' + colors.ENDC)  # noqa

            if not inp == "end" and inp != "quit":
                s += inp + "\n"
            elif inp == "end":
                s = s[:-1]
                break
            elif inp == "quit":
                print(colors.HEADER+"bye!"+colors.ENDC)
                exit(0)
        ast = parser.parse(s)
        html = renderer.render(ast)
        print(colors.WARNING+"="*10+"AST====="+colors.ENDC)
        CommonMark.dumpAST(ast)
        print(colors.WARNING+"="*10+"HTML===="+colors.ENDC)
        print(html)

# some tests?
if args.t:
    tests = args.t.split(",")
    choice_examples = []
    for t in tests:
        if not t == "" and len(examples) > int(t):
            choice_examples.append(examples[int(t)-1])
    examples = choice_examples

# all tests

for i, example in enumerate(examples):  # [0,examples[0]]
    if not example['section'] == "" and \
       not current_section == example['section']:
        print('\n' + colors.HEADER + '[' + example['section'] + ']' +
              colors.ENDC + ' ', end='')
        current_section = example['section']
        catStats.update({current_section: [0, 0, 0]})

    catStats[current_section][2] += 1
    if args.d:
        print(colors.HEADER+"[Parsing]"+colors.ENDC)
    ast = parser.parse(re.sub(tabChar, "\t", example['markdown']))
    if args.d:
        print(colors.HEADER+"[Rendering]"+colors.ENDC)
    actual = renderer.render(ast)
    if re.sub('\t', tabChar, actual) == example['html']:
        passed += 1
        catStats[current_section][0] += 1
        if not args.f:
            print(colors.OKGREEN + '✓' + colors.ENDC, end='')
        if args.d:
            CommonMark.dumpAST(ast)
        if args.p or args.d and not args.np:
            print(
                colors.OKBLUE +
                "=== markdown ===============\n" +
                colors.ENDC + showSpaces(example['markdown']) +
                colors.OKBLUE +
                "\n=== expected ===============\n" +
                colors.ENDC + showSpaces(example['html']) +
                colors.OKBLUE +
                "\n=== got ====================\n" +
                colors.ENDC + showSpaces(actual))
    else:
        failed += 1
        catStats[current_section][1] += 1
        if args.t:
            print("Test #" + str(args.t.split(",")[i]), end='')
        else:
            print("Test #" + str(i+1), end='')
        print(' ' + colors.FAIL + "✗" + colors.ENDC)
        if args.d:
            CommonMark.dumpAST(ast)
        if not args.np or args.f:
            print(
                colors.WARNING +
                "=== markdown ===============\n" +
                colors.ENDC + showSpaces(example['markdown']) +
                colors.WARNING +
                "\n=== expected ===============\n" +
                colors.ENDC + showSpaces(example['html']) +
                colors.WARNING +
                "\n=== got ====================\n" +
                colors.ENDC + showSpaces(actual))

print('\n' + str(passed) + ' tests passed, ' + str(failed) + ' failed')

endTime = time.clock()
runTime = endTime-startTime

if args.s:
    for i in catStats.keys():
        per = catStats[i][0]/catStats[i][2]
        print(colors.HEADER + "[" + i + "]" + colors.ENDC +
              "\t" + str(per*100) + "% Passed")

print("runtime: " + str(runTime) + "s")

if (failed > 0):
    sys.exit(1)



#!/usr/bin/env python
from __future__ import unicode_literals
import argparse
import sys
import CommonMark
parser = argparse.ArgumentParser(
    description="Process Markdown according to the CommonMark specification.")
if sys.version_info < (3, 0):
    reload(sys)
    sys.setdefaultencoding('utf-8')
parser.add_argument(
    'infile',
    nargs="?",
    type=argparse.FileType('r'),
    default=sys.stdin,
    help="Input Markdown file to parse, defaults to STDIN")
parser.add_argument(
    '-o',
    nargs="?",
    type=argparse.FileType('w'),
    default=sys.stdout,
    help="Output HTML/JSON file, defaults to STDOUT")
parser.add_argument('-a', action="store_true", help="Print formatted AST")
parser.add_argument('-aj', action="store_true", help="Output JSON AST")
args = parser.parse_args()
parser = CommonMark.Parser()
f = args.infile
o = args.o
lines = []
for line in f:
    lines.append(line)
data = "".join(lines)
ast = parser.parse(data)
if not args.a and not args.aj:
    renderer = CommonMark.HtmlRenderer()
    o.write(renderer.render(ast))
    exit()
if args.a:
    # print ast
    CommonMark.dumpAST(ast)
    exit()

# o.write(ast.to_JSON())
o.write(CommonMark.ASTtoJSON(ast))
exit()



#!/usr/bin/env python
# 2014 - Bibek Kafle & Roland Shoemaker
# Port of @jgm's JavaScript stmd.js implementation of the CommonMark spec

# Basic usage:
#
# import CommonMark
# parser = CommonMark.Parser()
# renderer = CommonMark.HtmlRenderer()
# print(renderer.render(parser.parse('Hello *world*')))
from __future__ import absolute_import, unicode_literals
import json
from builtins import str
from CommonMark.blocks import Parser
from CommonMark.html import HtmlRenderer


# Utility functions


def commonmark(text, format="html"):
    """Render CommonMark into HTML, JSON or AST
    Optional keyword arguments:
    format:     'html' (default), 'json' or 'ast'

    >>> commonmark("*hello!*")
    '<p><em>hello</em></p>\\n'
    """
    parser = Parser()
    ast = parser.parse(text)
    if format not in ["html", "json", "ast"]:
        raise ValueError("format must be 'html', 'json' or 'ast'")
    if format == "html":
        renderer = HtmlRenderer()
        return renderer.render(ast)
    if format == "json":
        return ASTtoJSON(ast)
    if format == "ast":
        return dumpAST(ast)


def prepare(block):
    """ Strips circular 'parent' references and trims empty
    block elements."""
    to_remove = [
        'parent', 'nxt', 'prv', 'first_child', 'last_child',
    ]
    for r in to_remove:
        block.__dict__[r] = None
    if block.is_open is not None:
        block.__dict__['open'] = block.is_open
        del(block.is_open)
    # trim empty elements...
    for attr in dir(block):
        if not callable(attr) and not attr.startswith("__") and \
           attr not in ['pretty', 'is_container',
                        'append_child', 'prepend_child', 'unlink',
                        'insert_after', 'insert_before', 'walker']:
            if block.__dict__[attr] in ["", [], None, {}]:
                del(block.__dict__[attr])
    return block


def ASTtoJSON(block):
    """ Output AST in JSON form, this is destructive of block."""
    # sort_keys=True) # indent=4)
    return json.dumps(prepare(block), default=lambda o: o.__dict__)


def dumpAST(obj, ind=0, topnode=False):
    """ Print out a block/entire AST."""
    indChar = ("\t" * ind) + "-> " if ind else ""
    print(indChar + "[" + obj.t + "]")
    if not obj.title == "":
        print("\t" + indChar + "Title: " + (obj.title or ''))
    if not obj.info == "":
        print("\t" + indChar + "Info: " + (obj.info or ''))
    if not obj.destination == "":
        print("\t" + indChar + "Destination: " + (obj.destination or ''))
    if obj.is_open:
        print("\t" + indChar + "Open: " + str(obj.is_open))
    if obj.last_line_blank:
        print(
            "\t" + indChar + "Last line blank: " + str(obj.last_line_blank))
    if obj.sourcepos:
        print("\t" + indChar + "Sourcepos: " + str(obj.sourcepos))
    if not obj.string_content == "":
        print("\t" + indChar + "String content: " + (obj.string_content or ''))
    if not obj.info == "":
        print("\t" + indChar + "Info: " + (obj.info or ''))
    if not obj.literal == "":
        print("\t" + indChar + "Literal: " + (obj.literal or ''))
    if obj.list_data.get('type'):
        print("\t" + indChar + "List Data: ")
        print("\t\t" + indChar + "[type] = " + obj.list_data.get('type'))
        if obj.list_data.get('bullet_char'):
            print(
                "\t\t" + indChar + "[bullet_char] = " +
                obj.list_data['bullet_char'])
        if obj.list_data.get('start'):
            print("\t\t" + indChar + "[start] = " + obj.list_data.get('start'))
        if obj.list_data.get('delimiter'):
            print(
                "\t\t" + indChar + "[delimiter] = " +
                obj.list_data.get('delimiter'))
        if obj.list_data.get('padding'):
            print(
                "\t\t" + indChar + "[padding] = " +
                str(obj.list_data.get('padding')))
        if obj.list_data.get('marker_offset'):
            print(
                "\t\t" + indChar + "[marker_offset] = " +
                str(obj.list_data.get('marker_offset')))
    if obj.walker:
        print("\t" + indChar + "Children:")
        walker = obj.walker()
        nxt = walker.nxt()
        while nxt is not None and topnode is False:
            dumpAST(nxt['node'], ind + 2, topnode=True)
            nxt = walker.nxt()



from __future__ import absolute_import, unicode_literals

import re
from builtins import str
from CommonMark.common import escape_xml


reHtmlTag = re.compile(r'\<[^>]*\>')
reUnsafeProtocol = re.compile(
    r'^javascript:|vbscript:|file:|data:', re.IGNORECASE)
reSafeDataProtocol = re.compile(
    r'^data:image\/(?:png|gif|jpeg|webp)', re.IGNORECASE)


def tag(name, attrs=[], selfclosing=False):
    """Helper function to produce an HTML tag."""
    result = '<' + name
    for attr in attrs:
        result += ' {}="{}"'.format(attr[0], attr[1])
    if selfclosing:
        result += ' /'

    result += '>'
    return result


def potentially_unsafe(url):
    return re.search(reUnsafeProtocol, url) and not \
        re.search(reSafeDataProtocol, url)


class HtmlRenderer:

    def __init__(self, options={}):
        # by default, soft breaks are rendered as newlines in HTML.
        # set to "<br />" to make them hard breaks
        # set to " " if you want to ignore line wrapping in source
        self.softbreak = '\n'
        self.options = options

    def out(self, s):
        if self.disable_tags > 0:
            self.buf += re.sub(reHtmlTag, '', s)
        else:
            self.buf += s
        self.last_out = s

    def cr(self):
        if self.last_out != '\n':
            self.buf += '\n'
            self.last_out = '\n'

    def renderNodes(self, block):
        walker = block.walker()
        self.buf = ''
        self.last_out = '\n'
        self.disable_tags = 0

        event = walker.nxt()
        while event is not None:
            entering = event['entering']
            node = event['node']

            attrs = []
            if self.options.get('sourcepos'):
                pos = node.sourcepos
                if pos:
                    attrs.push([
                        'data-sourcepos',
                        pos[0][0] + ':' + pos[0][1] + '-' +
                        pos[1][0] + ':' + pos[1][1]])

            if node.t == 'Text':
                self.out(escape_xml(node.literal, False))
            elif node.t == 'Softbreak':
                self.out(self.softbreak)
            elif node.t == 'Hardbreak':
                self.out(tag('br', [], True))
                self.cr()
            elif node.t == 'Emph':
                self.out(tag('em' if entering else '/em'))
            elif node.t == 'Strong':
                self.out(tag('strong' if entering else '/strong'))
            elif node.t == 'HtmlInline':
                if self.options.get('safe'):
                    self.out('<!-- raw HTML omitted -->')
                else:
                    self.out(node.literal)
            elif node.t == 'CustomInline':
                if entering and node.on_enter:
                    self.out(node.on_enter)
                elif not entering and node.on_exit:
                    self.out(node.on_exit)
            elif node.t == 'Link':
                if entering:
                    if not (self.options.get('safe') and
                            potentially_unsafe(node.destination)):
                        attrs.append([
                            'href',
                            escape_xml(node.destination, True)
                        ])
                    if node.title:
                        attrs.append(['title', escape_xml(node.title, True)])
                    self.out(tag('a', attrs))
                else:
                    self.out(tag('/a'))
            elif node.t == 'Image':
                if entering:
                    if self.disable_tags == 0:
                        if self.options.get('safe') and \
                           potentially_unsafe(node.destination):
                            self.out('<img src="" alt="')
                        else:
                            self.out(
                                '<img src="{}" alt="'.format(
                                    escape_xml(node.destination, True)))
                    self.disable_tags += 1
                else:
                    self.disable_tags -= 1
                    if self.disable_tags == 0:
                        if node.title:
                            self.out('" title="' +
                                     escape_xml(node.title, True))
                        self.out('" />')
            elif node.t == 'Code':
                self.out(
                    tag('code') +
                    escape_xml(node.literal, False) +
                    tag('/code'))
            elif node.t == 'Document':
                pass
            elif node.t == 'Paragraph':
                grandparent = node.parent.parent
                if grandparent is not None and \
                   grandparent.t == 'List' and \
                   grandparent.list_data.get('tight'):
                    pass
                else:
                    if entering:
                        self.cr()
                        self.out(tag('p', attrs))
                    else:
                        self.out(tag('/p'))
                        self.cr()
            elif node.t == 'BlockQuote':
                if entering:
                    self.cr()
                    self.out(tag('blockquote', attrs))
                    self.cr()
                else:
                    self.cr()
                    self.out(tag('/blockquote'))
                    self.cr()
            elif node.t == 'Item':
                if entering:
                    self.out(tag('li', attrs))
                else:
                    self.out(tag('/li'))
                    self.cr()
            elif node.t == 'List':
                tagname = 'ul' if node.list_data['type'] == 'Bullet' else 'ol'
                if entering:
                    try:
                        start = node.list_data['start']
                    except KeyError:
                        start = None
                    if start is not None and start != 1:
                        attrs.append(['start', str(start)])
                    self.cr()
                    self.out(tag(tagname, attrs))
                    self.cr()
                else:
                    self.cr()
                    self.out(tag('/' + tagname))
                    self.cr()
            elif node.t == 'Heading':
                tagname = 'h' + str(node.level)
                if entering:
                    self.cr()
                    self.out(tag(tagname, attrs))
                else:
                    self.out(tag('/' + tagname))
                    self.cr()
            elif node.t == 'CodeBlock':
                info_words = re.split(r'\s+', node.info) if node.info else []
                if len(info_words) > 0 and len(info_words[0]) > 0:
                    attrs.append([
                        'class',
                        'language-' + escape_xml(info_words[0], True)
                    ])
                self.cr()
                self.out(tag('pre') + tag('code', attrs))
                self.out(escape_xml(node.literal, False))
                self.out(tag('/code') + tag('/pre'))
                self.cr()
            elif node.t == 'HtmlBlock':
                if self.options.get('safe'):
                    self.out('<!-- raw HTML omitted -->')
                else:
                    self.out(str(node.literal))
                self.cr()
            elif node.t == 'CustomBlock':
                self.cr()
                if entering and node.on_enter:
                    self.out(node.on_enter)
                elif not entering and node.on_exit:
                    self.out(node.on_exit)
                self.cr()
            elif node.t == 'ThematicBreak':
                self.cr()
                self.out(tag('hr', attrs, True))
                self.cr()
            else:
                raise ValueError('Unknown node type {}'.format(node.t))
            event = walker.nxt()
        return self.buf

    render = renderNodes



# CommonMark.py - setup.py
from setuptools import setup, Command


class Test(Command):
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        import subprocess
        import sys
        errno = subprocess.call([
            sys.executable,
            'CommonMark/test/test-CommonMark.py'])
        raise SystemExit(errno)


setup(
    name="CommonMark",
    packages=['CommonMark'],
    scripts=['bin/cmark.py'],
    version="0.6.2",
    license="BSD License",
    description="Python parser for the CommonMark Markdown spec",
    author="Bibek Kafle <bkafle662@gmail.com>, " +
    "Roland Shoemaker <rolandshoemaker@gmail.com>",
    author_email="rolandshoemaker@gmail.com",
    maintainer="Nik Nyby",
    maintainer_email="nikolas@gnu.org",
    url="https://github.com/rtfd/CommonMark-py",
    keywords=["markup", "markdown", "commonmark"],
    cmdclass={'test': Test},
    install_requires=[
        'future',
    ],
    setup_requires=[
        'flake8',
    ],
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 3",
        "Development Status :: 4 - Beta",
        "Environment :: Other Environment",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Topic :: Documentation",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Software Development :: Documentation",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Text Processing :: Markup",
        "Topic :: Utilities"])



from __future__ import absolute_import, unicode_literals
__all__ = ["HtmlRenderer", "Parser", "dumpAST", "ASTtoJSON", "commonmark"]
from CommonMark.CommonMark import HtmlRenderer
from CommonMark.CommonMark import Parser
from CommonMark.CommonMark import dumpAST
from CommonMark.CommonMark import ASTtoJSON
from CommonMark.CommonMark import commonmark
